// Eseq2MidiDlg.cpp : implementation file
//

#include "stdafx.h"
#include "Eseq2Midi.h"
#include "Eseq2MidiDlg.h"
#include "AlertDlg.h"
#include "Midiio.hpp"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CEseq2MidiDlg dialog

CEseq2MidiDlg::CEseq2MidiDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CEseq2MidiDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CEseq2MidiDlg)
	m_eseq = _T("");
	m_midi = _T("");
	m_key = 0;
	//}}AFX_DATA_INIT
	// Note that LoadIcon does not require a subsequent DestroyIcon in Win32
	m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
}

void CEseq2MidiDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CEseq2MidiDlg)
	DDX_Control(pDX, IDC_CONVERT, m_convert);
	DDX_Control(pDX, IDC_MIDI, m_midiedit);
	DDX_Control(pDX, IDC_ESEQ, m_eseqedit);
	DDX_Control(pDX, IDC_MAJORMINOR, m_majorminor);
	DDX_Control(pDX, IDC_NUM, m_num);
	DDX_Control(pDX, IDC_DENOM, m_denom);
	DDX_Text(pDX, IDC_ESEQ, m_eseq);
	DDX_Text(pDX, IDC_MIDI, m_midi);
	DDX_Text(pDX, IDC_KEY, m_key);
	DDV_MinMaxInt(pDX, m_key, -6, 6);
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CEseq2MidiDlg, CDialog)
	//{{AFX_MSG_MAP(CEseq2MidiDlg)
	ON_WM_PAINT()
	ON_WM_QUERYDRAGICON()
	ON_BN_CLICKED(IDC_BROWSEESEQ, OnBrowseEseq)
	ON_BN_CLICKED(IDC_BROWSEMIDI, OnBrowseMidi)
	ON_BN_CLICKED(IDC_CONVERT, OnConvert)
	ON_EN_KILLFOCUS(IDC_TEMPO, OnKillfocusTempo)
	ON_EN_KILLFOCUS(IDC_KEY, OnKillfocusKey)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CEseq2MidiDlg message handlers

BOOL CEseq2MidiDlg::OnInitDialog()
{
	CDialog::OnInitDialog();

	// Set the icon for this dialog.  The framework does this automatically
	//  when the application's main window is not a dialog
	SetIcon(m_hIcon, TRUE);			// Set big icon
	SetIcon(m_hIcon, FALSE);		// Set small icon
	
	// TODO: Add extra initialization here
	m_num.SetCurSel(2);
	m_denom.SetCurSel(1);
	m_key = 0;
	m_majorminor.SetCurSel(0);

	UpdateData(FALSE);

	return TRUE;  // return TRUE  unless you set the focus to a control
}

// If you add a minimize button to your dialog, you will need the code below
//  to draw the icon.  For MFC applications using the document/view model,
//  this is automatically done for you by the framework.

void CEseq2MidiDlg::OnPaint() 
{
	if (IsIconic())
	{
		CPaintDC dc(this); // device context for painting

		SendMessage(WM_ICONERASEBKGND, (WPARAM) dc.GetSafeHdc(), 0);

		// Center icon in client rectangle
		int cxIcon = GetSystemMetrics(SM_CXICON);
		int cyIcon = GetSystemMetrics(SM_CYICON);
		CRect rect;
		GetClientRect(&rect);
		int x = (rect.Width() - cxIcon + 1) / 2;
		int y = (rect.Height() - cyIcon + 1) / 2;

		// Draw the icon
		dc.DrawIcon(x, y, m_hIcon);
	}
	else
	{
		CDialog::OnPaint();
	}
}

// The system calls this to obtain the cursor to display while the user drags
//  the minimized window.
HCURSOR CEseq2MidiDlg::OnQueryDragIcon()
{
	return (HCURSOR) m_hIcon;
}

void CEseq2MidiDlg::OnBrowseEseq() 
{
	int n;

	CFileDialog dlg(TRUE, _T("evt"), NULL,
		OFN_FILEMUSTEXIST | OFN_HIDEREADONLY,
		_T("Eseq files (*.evt)|*.evt|All Files (*.*)|*.*||"));
	if (dlg.DoModal() != IDOK)
		return;                 // stay with old data file

	m_eseq = dlg.GetPathName();	// fill in the eseq filename
	m_midi = m_eseq;
	
	n = m_midi.Replace(".evt", ".mid");
	if (n != 1)
	{
		CString ext = dlg.GetFileExt();
		m_midi.Replace(ext, "mid");
	}

	m_convert.SetFocus();

	UpdateData(FALSE);
}

void CEseq2MidiDlg::OnBrowseMidi() 
{
	CFileDialog dlg(TRUE, _T("mid"), NULL,
		OFN_FILEMUSTEXIST | OFN_HIDEREADONLY,
		_T("Eseq files (*.mid)|*.mid|All Files (*.*)|*.*||"));
	if (dlg.DoModal() != IDOK)
		return;                 // stay with old data file

	m_midi = dlg.GetPathName();
	UpdateData(FALSE);
}

void CEseq2MidiDlg::OnConvert() 
{
	// First get the info, entered by the user
	UpdateData(TRUE);
	AlertDlg alertdlg;

	if (m_eseq.IsEmpty())
	{
		alertdlg.m_alert = "Please enter an ESEQ filename";
		alertdlg.DoModal();
		m_eseqedit.SetFocus();
		return;
	}

	if (m_midi.IsEmpty())
	{
		alertdlg.m_alert = "Please enter an MIDI filename";
		alertdlg.DoModal();
		m_midiedit.SetFocus();
		return;
	}

	CString eseqname = m_eseq;
	CString midiname = m_midi;

	FILE* eseqf = fopen(eseqname, "rb");
	if (!eseqf)
	{
		alertdlg.m_alert.Format("Can't open file: %s", eseqname);
		alertdlg.DoModal();
		return;
	}
	int num = m_num.GetCurSel() + 2;
	int denom;
	switch (m_denom.GetCurSel())
	{
		case 0: denom = 2; break;
		case 1: denom = 4; break;
		case 2: denom = 8; break;
		case 3: denom = 16; break;
		default: denom = 4;
	}

//	int denom = *(m_denom.GetItemDataPtr(m_denom.GetCurSel()));
	int majorminor = m_majorminor.GetCurSel();


	char header[256];
	fread(header, sizeof(header), 1, eseqf);
	
	for (int idofs = 0; idofs < 16 ; idofs++)
		if (strncmp(header+idofs, "COM-ESEQ", 8) == 0)
			break;
		
	if (idofs >= 16)
	{
		alertdlg.m_alert.Format("%s is not an ESEQ file", eseqname);
		alertdlg.DoModal();
		fclose(eseqf);
		return;
	}
	
//	in ESEQ files the bpm is always 156
	int bpm = header[idofs+29] + 29;  // ususally +30?

	int c;
	
	while ((c = fgetc(eseqf)) != EOF && c != 0xF1)
		;

	if (c == EOF)
	{
		alertdlg.m_alert.Format("%s: start not found", eseqname);
		alertdlg.DoModal();
		fclose(eseqf);
		return;
	}

	c = fgetc(eseqf); // 0

	MidiWrite midi(midiname);
	if (!midi.getf())
	{
		alertdlg.m_alert.Format("Can't open file: %s", midiname);
		alertdlg.DoModal();
		fclose(eseqf);
		return;
	}
	midi.head(VERSION_MULTICHANNEL, 0, 384);
	midi.track();
	midi.tact(num, denom, 24, 8);
	midi.tempo(ticks(bpm));
	midi.key(m_key, majorminor);
	
	while ((c = fgetc(eseqf)) != EOF && c != 0xf2)
	{
		if (c == 0xf3)
		{
			c = fgetc(eseqf);

			midi.time(c);
			continue;
		}
		if (c == 0xfe)
		{
			c = fgetc(eseqf); // 0x78
			continue;
		}
		if (c == 0xf4)
		{
			c = fgetc(eseqf);
			midi.time(c);
			c = fgetc(eseqf);
			midi.time(128L * c);
			continue;
		}
		if (c == 0xf0)
		{
			LPBYTE pSysex;
			DWORD cbDataSize;
			DWORD cbNumBytes;

			cbDataSize = 8192;
			cbNumBytes = 0;

			pSysex = (LPBYTE)GlobalLock(GlobalAlloc(GHND, cbDataSize));
			pSysex[0] = c;
			cbNumBytes++;

			while ((c = fgetc(eseqf)) != EOF && c != 0xf7)
			{
				if (cbNumBytes == cbDataSize)
				{
					// grow the buffer in 8K blocks
					cbDataSize += 8192;
					pSysex = (LPBYTE)GlobalLock(GlobalReAlloc(GlobalHandle(pSysex), cbDataSize, GMEM_MOVEABLE));

					// Did the realloc succeed?
					if (pSysex == NULL)
					{
						// Very bad news. Discard everything!
						cbDataSize = cbNumBytes = 0;
					}
				}
				pSysex[cbNumBytes] = c;

				// Next position in buffer
				cbNumBytes++;
			}

			if (c = 0xF7)
			{
				pSysex[cbNumBytes] = c;
				cbNumBytes++;
			}

			midi.sysex(1, pSysex);

			// free the sysex buffer again
			GlobalFree(GlobalHandle(pSysex));
			continue;
		}
		if (c >= 0x80 && c <= 0xef)
		{
			unsigned char param[2];
			if (c >= 0xc0 && c <= 0xdf)
			{
				// 1 byte
				param[0] = fgetc(eseqf);
				midi.event(c, 1, param);
			}
			else
			{
				// 2 byte
				param[0] = fgetc(eseqf);
				param[1] = fgetc(eseqf);
				midi.event(c, 2, param);
			}
			continue;
		}
		alertdlg.m_alert.Format("Warning: unknown event %02X ignored!", c);
		alertdlg.DoModal();
	}
	midi.endtrack();

	alertdlg.m_alert = "Conversion successfully completed";
	alertdlg.DoModal();
	
	fclose(eseqf);
	return;
}


void CEseq2MidiDlg::OnKillfocusTempo() 
{
	UpdateData(TRUE);	/* value verification */	
}

void CEseq2MidiDlg::OnKillfocusKey() 
{
	UpdateData(TRUE);	/* value verification */
}
